# -*- coding: utf-8 -*-
import xml.etree.cElementTree as ET
import os
import csv

os.chdir('C:\\Users\\christoff.jorgensen\\Documents\\Master\\Data fra nett\\klima')

def open_xml(filename):
    """Opens a xml-file, and returns it as a string."""
    with open(filename, 'r') as f:
        data = f.read()
    return data

def parser(data):
    """parses the xml-files to a elementTree"""
    root = ET.fromstring(data)
    return root


def get_table(root, stationname, descriptor, year):
    """returns a table for a certain station and descriptor"""
    """descriptor is either RR (downpour) or TAM (temperature)"""
    data = []
    for table in root.iter('table'):
        if ((table.get('name').find(stationname) >= 0) and
            (table.get('name').find(descriptor) >= 0) and
            (table.get('name').find(year) >= 0)):
            """searches for the right table"""
            data = table
            break
            """breaks loop when correct table is found."""
        
    return data


def get_values(table, city, descriptor, year):
    """Get values for each day in each month over a year"""
    table = get_table(table, city, descriptor, year)
    """Gets the correct table from the xml-file"""

    if city == "KJEVIK":
        city = "KRISTIANSAND"
        """Quick fix due to different station name in xml-file for TAM krSand."""
    
    values = [[],[],[],[],[],[],[],[],[],[],[],[]]
    for dag in table.iter('Dag'):
        i = 0
        dayNum = dag.get('id').strip('ID ')
        while i < 12:
            """iterates over the months in a year"""
            values[i].append((city, int(year), i+1, int(dayNum), dag[i].text))
            i += 1
            
        if int(dayNum) == 31:
            break
            """breaks when all days have been process due to excessive data in xml-files."""
        

    return values

def table_to_text(name, table):
    """Prints a table to a text file. Requires a table"""
    name = name + '.csv'
    with open(name, 'w') as f:
        i = 0
        while i < len(table):
            f.write(str(table[i]) + '\n')
            i += 1


def merge_city_data(city, rr13, rr14, tam13, tam14):
    """Merges data from different files for a city."""
    """Returns a file containg all weather data for the selected days for each city."""
    all_info = []
    i = 0
    while i < 12:
        for index, value in enumerate(rr13[i]):
            split_rr13 = str(value).strip('(').strip(')').split(',')
            split_tam13 = str(tam13[i][index]).strip('(').strip(')').split(',')
            
            year = int(split_rr13[1])
            month = int(split_rr13[2])
            day = int(split_rr13[3])
            rain = str(split_rr13[4]).strip(" '").strip("'")
            temp = str(split_tam13[4]).strip(" '").strip("'")
            
            if (month < 4):
                """data from 10th of april 2013 and onwards"""
                continue
            elif ((month == 4) and (day < 9)):
                continue
            
            all_info.append((year, month, day, temp, rain))
            
        i += 1

    j = 0
    while j < 4:
        """Just want all data up to 9th of april 2014."""
        for index, value in enumerate(rr14[j]):
            split_rr14 = str(value).strip('(').strip(')').split(',')
            split_tam14 = str(tam13[j][index]).strip('(').strip(')').split(',')
            
            year = int(split_rr14[1])
            month = int(split_rr14[2])
            day = int(split_rr14[3])
            rain = str(split_rr14[4]).strip(" '").strip("'")
            temp = str(split_tam14[4]).strip(" '").strip("'")

            if ((month == 4) and (day >= 10)):
                """if we get past 9th of april, increment j and break while loop."""
                j = 4
                break
                
            all_info.append((year, month, day, temp, rain))

        j += 1

    table_to_text(city, all_info) 

def magic():
    """does the magic"""
    """Data for all except TAM Kristiansand and RR OSLO"""
    data = open_xml('alle.xml')
    root = parser(data)
    
    oslo_TAM_2013 = get_values(root, 'OSLO', 'TAM', '2013')
    oslo_TAM_2014 = get_values(root, 'OSLO', 'TAM', '2014')

    trHeim_RR_2013 = get_values(root, 'TRONDHEIM', 'RR', '2013')
    trHeim_RR_2014 = get_values(root, 'TRONDHEIM', 'RR', '2014')
    trHeim_TAM_2013 = get_values(root, 'TRONDHEIM', 'TAM', '2013')
    trHeim_TAM_2014 = get_values(root, 'TRONDHEIM', 'TAM', '2014')

    krSand_RR_2013 = get_values(root, 'KRISTIANSAND', 'RR', '2013')
    krSand_RR_2014 = get_values(root, 'KRISTIANSAND', 'RR', '2014')


    """Data for TAM Kristiansand."""
    data_Kjevik = open_xml('temp_KJEVIK.xml')
    root_Kjevik = parser(data_Kjevik)
    
    krSand_TAM_2013 = get_values(root_Kjevik, 'KJEVIK', 'TAM', '2013')
    krSand_TAM_2014 = get_values(root_Kjevik, 'KJEVIK', 'TAM', '2014')


    """Data for RR Oslo."""
    data_OSLO_RR = open_xml('oslo_ny_nedbor.xml')
    root_OSLO_RR = parser(data_OSLO_RR)

    oslo_RR_2013 = get_values(root_OSLO_RR, 'OSLO', 'RR', '2013')
    oslo_RR_2014 = get_values(root_OSLO_RR, 'OSLO', 'RR', '2014')

    merge_city_data('OSLO', oslo_RR_2013, oslo_RR_2014,
                    oslo_TAM_2013, oslo_TAM_2013)
    
    merge_city_data('TRONDHEIM', trHeim_RR_2013, trHeim_RR_2014,
                    trHeim_TAM_2013, trHeim_TAM_2014)
    
    merge_city_data('KRISTIANSAND', krSand_RR_2013, krSand_RR_2014,
                    krSand_TAM_2013, krSand_TAM_2014)
    
    
    
    
    
    
