import os
import csv
from math import log

os.chdir('C:\Users\christoff.jorgensen\Downloads')

def parse(filename):
    table = []
    filename = filename + '.csv'
    with open(filename) as f:
        for line in f:
            table.append(line.strip('\n'))
    table.pop(-1) """Dummy to remove headers"""
    return table

def splitTable(table):
    """creates a new table with wanted values"""
    new_table = []
    for line in table:
        split = line.split(',')
        ID = split[0].strip('(').strip('"')
        aNode = split[1].strip('"')
        bNode = split[2].strip('"')
        dist = split[3].strip('"')
        slope_AB = split[13].strip('"')
        new_table.append((ID, aNode, bNode, dist, slope_AB))
    new_table.pop(0)
    return new_table

def next_segment(table, start, sortTable):
    """Recursive function to get next segment in a route"""
    """requires the main table, a start node value and and empty table"""
    start = int(start)
    for index, value in enumerate(table):
        ID, aNode, bNode, dist, slope_AB = value
        ID = int(ID)
        aNode = int(aNode)
        bNode = int(bNode)
        dist = int(dist)
        slope_AB = int(slope_AB)
        if start == aNode:
            table.pop(index)
            sortTable.append((ID, aNode, bNode, dist, slope_AB))
            if len(table) > 0:
                return(next_segment(table, bNode, sortTable))
            else:
                return sortTable
        elif start == bNode:
            table.pop(index)
            sortTable.append((ID, bNode, aNode, dist, -slope_AB))
            if len(table) > 0:
                return(next_segment(table, aNode, sortTable))
            else:
                return sortTable

def neg(c_slope, c_len, p_slope):
    """function for negative slopse"""
    v = 66.73 - 0.86*c_slope + 6.85*log(c_len, 10) - 31.1 * log(p_slope + 36, 10)
    return v

def flat(c_slope, c_len, p_slope, p_len, n_slope, n_len):
    """function for flat segments"""
    v = 12.6 - 2.4*c_slope + 2.44*log(c_len, 10) - 0.2*p_slope + 2.22*log(p_len, 10) - 0.18*n_slope + 1.47*log(n_len)
    return v

def pos(c_slope, c_len, p_slope, p_len, n_slope, n_len):
    """function for positive slopes"""
    v = 20.25 - c_slope - 1.1*log(c_len, 10) - 0.27*p_slope + 2.5*log(p_len, 10) - 0.25*n_slope + 0.87*log(n_len)
    return v

def getSpeed(c_slope, c_len, p_slope, p_len, n_slope, n_len):
    """Calculates speed based on slopes and lengths"""
    if c_slope < -2.0:
        v = neg(c_slope, c_len, p_slope)

    elif c_slope > 2.0:
        v = pos(c_slope, c_len, p_slope, p_len, n_slope, n_len)

    else:
        v = flat(c_slope, c_len, p_slope, p_len, n_slope, n_len)
    if v < 3:
        v = 3
    return v

def calculate(table):
    """Calculates traveltime on each segment."""
    """Requires a table with sorted values"""
    travelTimes = []
    for index, value in enumerate(table):
        ID, startNode, endNode, dist, slope = value
        ID = int(ID)
        startNode = int(startNode)
        endNode = int(endNode)
        dist = int(dist)
        slope = int(slope)
        if index == 0:
            prevSlope = 0
            prevLen = 1

            nextID, nextStartNode, nextEndNode, nextLen, nextSlope = table[index+1]
            nextLen = int(nextLen)
            nextSlope = int(nextSlope)
                            
        elif index == len(table)-1:
            nextSlope = 0
            nextLen = 1
                            
            prevID, prevStartNode, prevEndNode, prevLen, prevSlope = table[index-1]
            prevLen = int(prevLen)
            prevSlope = int(prevSlope)
                            
        else:
            prevID, prevStartNode, prevEndNode, prevLen, prevSlope = table[index-1]
            prevLen = int(prevLen)
            prevSlope = int(prevSlope)

            nextID, nextStartNode, nextEndNode, nextLen, nextSlope = table[index+1]
            nextLen = int(nextLen)
            nextSlope = int(nextSlope)
            
        speed = getSpeed(slope, dist, prevSlope, prevLen, nextSlope, nextLen)
        time_old = (dist*3.6/15)/60
        time_new = (dist*3.6/speed)/60
        travelTimes.append((ID, startNode, endNode, dist, slope, speed, '15', time_new, time_old))
    return travelTimes

def reverseTable(table):
    """Reverses a table for the return trip"""
    returnTrip = []
    for line in table:
        ID, endNode, startNode, dist, slope = line
        ID = int(ID)
        endNode = int(endNode)
        startNode = int(startNode)
        dist = int(dist)
        slope = -int(slope)
        returnTrip.insert(0, (ID, startNode, endNode, dist, slope))
    return returnTrip

def to_csv(table, filename):
    """Prints a table to .csv. Works well for further analysis in excel"""
    filename += '.csv'
    with open(filename, 'wb') as csvfile: 
        export = csv.writer(csvfile, dialect='excel-tab')
        export.writerows(table)

        
def route():
    """main function."""
    filename = raw_input("rute: ")
    table = parse(filename)
    split = splitTable(table)
    initialNode = raw_input("Start node: ")
    sortedTable = []
    sortedTable = next_segment(split, initialNode, sortedTable)
    returnTrip = reverseTable(sortedTable)
    trip = calculate(sortedTable)
    returnTrip = calculate(returnTrip)
    tripName = filename + '_trip'
    returnName = filename + '_returnTrip'
    to_csv(trip, tripName)
    to_csv(returnTrip, returnName)

    
    
    
    
    
    
       

